; ==================================================================================================
; Title:   MSChartDemo.inc
; Author:  G. Friedrich  @ July 2003
; Version: 1.0.0
; Purpose: MSChartDemo for ObjAsm32.
; ==================================================================================================


CLSSTYLE equ CS_BYTEALIGNWINDOW or CS_BYTEALIGNCLIENT or CS_VREDRAW or CS_HREDRAW; or CS_DROPSHADOW

CStr szMSChartDemoApp, "MSChartDemoApp"                 ;Creates szMSChartDemoApp string in .const


Object MSChartDemoApp, MySdiAppID, SdiApp               ;Single Document Interface App.
  RedefineMethod    Done
  RedefineMethod    Init                                ;Init method refefinition
  StaticMethod      Startup

  DynamicEvent      OnCommand,  WM_COMMAND
  VirtualEvent      OnClose,    WM_CLOSE, WM_QUERYENDSESSION
  VirtualEvent      OnPaint,    WM_PAINT
  VirtualEvent      OnSize,     WM_SIZE

  Embed Chart, MSChart

ObjectEnd                                               ;Ends object definition


Object MSChartAxisFormatDlg, , DialogModal
  RedefineMethod    CtlsGet
  RedefineMethod    CtlsSet
  RedefineMethod    Init,             Pointer, dword
  RedefineMethod    OnCommand,        dword, dword
  RedefineMethod    OnDestroy,        dword, dword

  DefineVariable    dAxisID,          dword,      0
ObjectEnd


.code

if IMPLEMENT

; ==================================================================================================
;    IMSChartEvents redefinition
; ==================================================================================================

; 
; Method:    IMSChartEvents.AxisLabelActivated2
; Purpose:   MSChart event handler.
; Arguments: Arg1: [in] -> Axis ID.
;            Arg2: [in] -> Axis Index.
;            Arg3: [in] -> Labelset Index.
;            Arg4: [in] -> Label Index.
;            Arg5: [in] -> Mouse flags.
;            Arg6: [out] -> Cancel flag.
; Return:    HRESULT.

Method IMSChartEvents.AxisLabelActivated2, uses esi, pAxisID:ptr sword, pAxisIndex:ptr sword, pLabelsetIndex:ptr sword, pLabelIndex:ptr sword, pMouseFlags:ptr sword, pCancel:ptr sword
    local AxisFormatDlg:MSChartAxisFormatDlg

    SetObject esi
    DbgText "IMSChartEvents.AxisLabelActivated"
    LNew AxisFormatDlg, MSChartAxisFormatDlg
    mov eax, [esi].pOwner
    mov ecx, pAxisID
    movzx edx, word ptr [ecx]
    OCall AxisFormatDlg::MSChartAxisFormatDlg.Init, [eax].Component.pOwner, edx
    OCall AxisFormatDlg::MSChartAxisFormatDlg.Show
    OCall AxisFormatDlg::MSChartAxisFormatDlg.Done
    mov eax, S_OK
MethodEnd

;; 
;; Method:    IMSChartEvents.DblClick2
;; Purpose:   MSChart event handler.
;; Arguments: None.
;; Return:    HRESULT.
;
;Method IMSChartEvents.DblClick2, uses ebx
;    local CRect:RECT
;
;    DbgText "IMSChartEvents.DblClick"
;    mov ecx, Self
;    mov edx, [ecx].IMSChartEvents.pOwner                  ;pChartSink
;    mov ebx, [edx].Component.pOwner                       ;MSChart
;    invoke GetClientRect, [ebx].MSChart.hWnd, addr CRect
;    ;This call to show the chart properties oly work if the container is in design mode.
;    ;Design mode is checked when the control is created inspecting the container.
;    ICall [ebx].MSChart.pIOleObject::IOleObject.DoVerb, 1, NULL, NULL, 0, [ebx].MSChart.hWnd, addr CRect
;    DbgComError eax
;    mov eax, S_OK
;MethodEnd


; ==================================================================================================
;    MSChartDemoApp implementation
; ==================================================================================================

; 
; Method:    MSChartDemoApp.Done
; Purpose:   Finalizes the SDI application object.
; Arguments: None.
; Return:    Nothing.

Method MSChartDemoApp.Done, uses esi
    SetObject esi
    OCall [esi].Chart::MSChart.Done
    ACall esi.Done
MethodEnd

; 
; Method:    MSChartDemoApp.Init
; Purpose:   Initalizes the SDI application object.
; Arguments: None.
; Return:    Nothing.

Method MSChartDemoApp.Init, uses ebx esi
    local dWwd:dword, dWht:dword
    local dWtx:dword, dWty:dword

    local pIVcPlot:Pointer, pIVcTitle:Pointer
    local pIVcAxis:Pointer, vArg:VARIANT, pIVcAxisTitle:Pointer
    local pIVcFont:Pointer

    local vChartArray:VARIANT, vChartDataX:VARIANT, vChartDataY:VARIANT
    local SAB2:SAFEARRAYBOUND, SAB1:SAFEARRAYBOUND      ;Don't change this order!
    local dIndex2:dword, dIndex1:dword                  ;Don't change this order!

    SetObject esi
    ACall esi.Init

    mov dWwd, 500                                       ;Default creation width
    mov dWht, 450                                       ;Default creation height
    mov dWtx, $invoke(CenterForm, dWwd, $invoke(GetSystemMetrics, SM_CXSCREEN)) ;Center the form
    mov dWty, $invoke(CenterForm, dWht, $invoke(GetSystemMetrics, SM_CYSCREEN)) ;Center the form

    invoke CreateWindowEx, WS_EX_LEFT or WS_EX_APPWINDOW, \
           offset szMSChartDemoApp, offset szAppTitle, WS_OVERLAPPEDWINDOW, \
           dWtx, dWty, dWwd, dWht, NULL, NULL, hInstance, esi ;Pass the instance pointer

    OCall [esi].Chart::MSChart.Init, esi
    .if eax == OBJ_OK
      ICall [esi].Chart.pIChart::_DMSChart.put_Repaint, FALSE
;      ICall [esi].Chart.pIChart::_DMSChart.put_DrawMode, VtChDrawModeBlit

      ;We redefine the AxisLabelActivated method of this MSChart instance
      ;  to provide a new functionality... Axis scale setting.
      Override [esi].Chart.pIChartEvents::IMSChartEvents.AxisLabelActivated, \
                                          IMSChartEvents.AxisLabelActivated2

      ;To define the data of the Chart, we have to pass to the Chart object a
      ;  SAFEARRAY of VARIANTS containing the data (real4)
      and SAB1.lLbound, 0
      mov SAB1.cElements, 10
      and SAB2.lLbound, 0
      mov SAB2.cElements, 4
      invoke SafeArrayCreate, VT_VARIANT, 2, addr SAB1    ;SafeArrayDestroy
      mov vChartArray.VARIANT.vt, VT_ARRAY or VT_VARIANT
      mov vChartArray.VARIANT.pArray, eax

      mov vChartDataX.VARIANT.vt, VT_I4
      mov vChartDataY.VARIANT.vt, VT_I4

      xor ebx, ebx
      .while ebx < 10
        mov dIndex1, ebx

        ;First line - red
        mov dIndex2, 0
        mov vChartDataX.VARIANT.lVal, ebx
        invoke SafeArrayPutElement, vChartArray.VARIANT.pArray, addr dIndex1, addr vChartDataX

        mov dIndex2, 1
        mov vChartDataY.VARIANT.lVal, ebx
        invoke SafeArrayPutElement, vChartArray.VARIANT.pArray, addr dIndex1, addr vChartDataY

        ;Second line - blue
        mov dIndex2, 2
        mov vChartDataX.VARIANT.lVal, ebx
        invoke SafeArrayPutElement, vChartArray.VARIANT.pArray, addr dIndex1, addr vChartDataX

        mov dIndex2, 3
        lea eax, [2*ebx]
        mov vChartDataY.VARIANT.lVal, eax
        invoke SafeArrayPutElement, vChartArray.VARIANT.pArray, addr dIndex1, addr vChartDataY

        inc ebx
      .endw

      ICall [esi].Chart.pIChart::_DMSChart.put_ChartData, vChartArray
      invoke SafeArrayDestroy, vChartArray.VARIANT.pArray

      ;Now we do a bit of customization of our chart
      ICall [esi].Chart.pIChart::_DMSChart.put_chartType, VtChChartType2dXY  ;VtChChartType3dLine
;      ICall [esi].Chart.pIChart::_DMSChart.put_AllowSelections, FALSE

      ICall [esi].Chart.pIChart::_DMSChart.put_TitleText, $OfsCBStr("Water flow")

      ICall [esi].Chart.pIChart::_DMSChart.get_Title, addr pIVcTitle
      ICall pIVcTitle::IVcTitle.get_VtFont, addr pIVcFont
      ICall pIVcFont::IVcFont.put_Size, 12.0
      ICall pIVcFont::IVcFont.put_Style, VtFontStyleBold
      ICall pIVcFont::IVcFont.put_Effect, VtFontEffectUnderline
      ICall pIVcFont::IVcFont.Release
      ICall pIVcTitle::IVcTitle.Release

      ICall [esi].Chart.pIChart::_DMSChart.get_Plot, addr pIVcPlot
      ICall pIVcPlot::IVcPlot.put_UniformAxis, FALSE

      ICall pIVcPlot::IVcPlot.get_Axis, VtChAxisIdX, vArg, addr pIVcAxis    ;vArg = dummy
      ICall pIVcAxis::IVcAxis.get_AxisTitle, addr pIVcAxisTitle
      ICall pIVcAxisTitle::IVcAxisTitle.put_Text, $OfsCBStr("Time")
      ICall pIVcAxisTitle::IVcAxisTitle.get_VtFont, addr pIVcFont
      ICall pIVcFont::IVcFont.put_Size, 10.0
      ICall pIVcFont::IVcFont.Release
      ICall pIVcAxisTitle::IVcAxisTitle.Release
      ICall pIVcAxis::IVcAxis.Release

      ICall pIVcPlot::IVcPlot.get_Axis, VtChAxisIdY, vArg, addr pIVcAxis    ;vArg = dummy
      ICall pIVcAxis::IVcAxis.get_AxisTitle, addr pIVcAxisTitle
      ICall pIVcAxisTitle::IVcAxisTitle.put_Text, $OfsCBStr("m^3/hs")
      ICall pIVcAxisTitle::IVcAxisTitle.get_VtFont, addr pIVcFont
      ICall pIVcFont::IVcFont.put_Size, 10.0
      ICall pIVcFont::IVcFont.Release
      ICall pIVcAxisTitle::IVcAxisTitle.Release
      ICall pIVcAxis::IVcAxis.Release

      ICall pIVcPlot::IVcPlot.Release

      ICall [esi].Chart.pIChart::_DMSChart.put_Repaint, TRUE
      ICall [esi].Chart.pIChart::_DMSChart.Refresh

    .else
      @invoke MessageBox, 0, "MSChart can't initialize", "Error", MB_OK or MB_ICONERROR
    .endif
    invoke ShowWindow, [esi].hWnd, SW_SHOWNORMAL        ;Show the window
    invoke UpdateWindow, [esi].hWnd                     ;Update if necessary
MethodEnd

; 
; Method:    MSChartDemoApp.OnCommand
; Purpose:   Event procedure for WM_COMMAND message.
; Arguments: Arg1: First message parameter.
;            Arg2: Second message parameter.
; Return:    Zero if handled.

Method MSChartDemoApp.OnCommand, uses esi, wParam:dword, lParam:dword
    local bBuffer[MAX_PATH]:byte, Dlg:DialogAbout

    SetObject esi
    mov eax, wParam
    .if ax == IDM_EXIT
      invoke SendMessage, [esi].hWnd, WM_SYSCOMMAND, SC_CLOSE, NULL
      xor eax, eax

    .elseif ax == IDM_ABOUT
      LNew Dlg, DialogAbout, Init, esi, [esi].hWnd
      OCall Dlg::DialogAbout.Show
      OCall Dlg::DialogAbout.Done
      xor eax, eax

    .elseif ax == IDM_HELP
      @invoke GetEnvironmentVariable, "OA32_PATH", addr bBuffer, sizeof bBuffer
      @invoke StrLCat, addr bBuffer, "\Help\ObjAsm32.chm", sizeof bBuffer
      invoke HtmlHelp, [esi].hWnd, addr bBuffer, HH_DISPLAY_TOPIC, 0
      xor eax, eax
    .else
      invoke DefWindowProc, [esi].hWnd, WM_COMMAND, wParam, lParam
    .endif
MethodEnd

; 
; Method:    MSChartDemoApp.OnClose
; Purpose:   Event procedure for WM_CLOSE message.
; Arguments: Arg1: First message parameter.
;            Arg2: Second message parameter.
; Return:    Zero if handled.

Method MSChartDemoApp.OnClose, uses esi, wParam:dword, lParam:dword
    SetObject esi
    @invoke MessageBox, [esi].hWnd, "Are you sure ?", "Application exit", MB_YESNO or MB_ICONQUESTION
    .if eax == IDNO
      xor eax, eax
    .else
      invoke DefWindowProc, [esi].hWnd, WM_CLOSE, wParam, lParam
    .endif
MethodEnd

; 
; Method:    MSChartDemoApp.OnPaint
; Purpose:   Event procedure for WM_PAINT message.
; Arguments: Arg1: First message parameter.
;            Arg2: Second message parameter.
; Return:    Zero if handled.

Method MSChartDemoApp.OnPaint, uses esi, wParam:dword, lParam:dword
    local PS:PAINTSTRUCT, hDC:Handle, Rct:RECT

    SetObject esi
    mov hDC, $invoke(BeginPaint, [esi].hWnd, addr PS)

    invoke GetClientRect, [esi].hWnd, addr Rct
    invoke DrawEdge, hDC, addr Rct, EDGE_SUNKEN, BF_RECT

    invoke EndPaint, [esi].hWnd, addr PS
    mov eax, TRUE
MethodEnd

; 
; Method:    MSChartDemoApp.OnSize
; Purpose:   Event procedure for WM_SIZE message.
; Arguments: Arg1: First message parameter.
;            Arg2: Second message parameter.
; Return:    Zero if handled.

Method MSChartDemoApp.OnSize, uses esi, wParam:dword, lParam:dword
    SetObject esi
    mov eax, lParam
    mov ecx, eax
    and eax, 0000FFFFh
    shr ecx, 16
    invoke MoveWindow, [esi].Chart.hWnd, 0, 0, eax, ecx, TRUE
MethodEnd

; 
; Method:    MSChartDemoApp.Startup
; Purpose:   Registers the object class with the OS.
; Arguments: None.
; Return:    Nothing.

Method MSChartDemoApp.Startup
    local WC:WNDCLASSEX

    mov WC.cbSize, sizeof WNDCLASSEX
    mov WC.style, CLSSTYLE
    m2m WC.lpfnWndProc, $MethodAddr(MSChartDemoApp.WndProc), edx
    and WC.cbClsExtra, 0
    and WC.cbWndExtra, 0
    m2m WC.hInstance, hInstance, edx
    mov WC.hbrBackground, 0
    mov WC.lpszMenuName, $OfsCStr("MENU_APP")
    mov WC.lpszClassName, offset szMSChartDemoApp
    mov WC.hIcon, $invoke(LoadIcon, hInstance, "ICON_APP_32")
    mov WC.hCursor, $invoke(LoadCursor, NULL, IDC_ARROW)
    and WC.hIconSm, 0

    invoke RegisterClassEx, addr WC
MethodEnd


; ==================================================================================================
;    MSChartAxisFormatDlg implementation
; ==================================================================================================

; 
; Method:    MSChartAxisFormatDlg.CtlsGet
; Purpose:   Reads the setting of all controls.
; Arguments: None.
; Return:    Nothing.

Method MSChartAxisFormatDlg.CtlsGet, uses esi
    local pIVcPlot:Pointer, vArg:VARIANT, pIVcAxis:Pointer, pIVcValueScale:Pointer
    local bNumberBuffer[100]:byte

    SetObject esi
    mov ecx, [esi].pOwner
    ICall [ecx].MSChart.pIChart::_DMSChart.get_Plot, addr pIVcPlot
    ICall pIVcPlot::IVcPlot.get_Axis, [esi].dAxisID, vArg, addr pIVcAxis
    ICall pIVcAxis::IVcAxis.get_ValueScale, addr pIVcValueScale

    invoke GetDlgItemText, [esi].hWnd, 1002, addr bNumberBuffer, sizeof bNumberBuffer
    invoke StrToSt0, addr bNumberBuffer
    sub esp, 8
    fstp real8 ptr [esp]
    ICall pIVcValueScale::IVcValueScale.put_Auto, FALSE
    ICall pIVcValueScale::IVcValueScale.put_Minimum, /real8

    invoke GetDlgItemText, [esi].hWnd, 1003, addr bNumberBuffer, sizeof bNumberBuffer
    invoke StrToSt0, addr bNumberBuffer
    sub esp, 8
    fstp real8 ptr [esp]
    ICall pIVcValueScale::IVcValueScale.put_Auto, FALSE
    ICall pIVcValueScale::IVcValueScale.put_Maximum, /real8

    ICall pIVcValueScale::IUnknown.Release
    ICall pIVcAxis::IUnknown.Release
    ICall pIVcPlot::IUnknown.Release

    mov ecx, [esi].pOwner
    ICall [ecx].MSChart.pIChart::_DMSChart.Refresh

MethodEnd

; 
; Method:    MSChartAxisFormatDlg.CtlsSet
; Purpose:   Sets the controls of the setup dialog.
; Arguments: None.
; Return:    This procedure should return TRUE to direct Windows to set the keyboard focus to
;            the control given by hwndFocus. Otherwise, it should return FALSE to prevent
;            Windows from setting the default keyboard focus.

Method MSChartAxisFormatDlg.CtlsSet, uses esi
    local pIVcPlot:Pointer, vArg:VARIANT, pIVcAxis:Pointer, pIVcValueScale:Pointer
    local r8ScaleVal:real8, bNumberBuffer[100]:byte

    SetObject esi
    mov ecx, [esi].pOwner
    ICall [ecx].MSChart.pIChart::_DMSChart.get_Plot, addr pIVcPlot
    ICall pIVcPlot::IVcPlot.get_Axis, [esi].dAxisID, vArg, addr pIVcAxis
    ICall pIVcAxis::IVcAxis.get_ValueScale, addr pIVcValueScale

    ICall pIVcValueScale::IVcValueScale.get_Minimum, addr r8ScaleVal
    fld r8ScaleVal
    invoke St0ToStr, addr bNumberBuffer, 0, 8, f_NOR or f_TRIM
    fUnload
    invoke SetDlgItemText, [esi].hWnd, 1002, addr bNumberBuffer

    ICall pIVcValueScale::IVcValueScale.get_Maximum, addr r8ScaleVal
    fld r8ScaleVal
    invoke St0ToStr, addr bNumberBuffer, 0, 8, f_NOR or f_TRIM
    fUnload
    invoke SetDlgItemText, [esi].hWnd, 1003, addr bNumberBuffer

    ICall pIVcValueScale::IUnknown.Release
    ICall pIVcAxis::IUnknown.Release
    ICall pIVcPlot::IUnknown.Release

    mov eax, TRUE
MethodEnd

; 
; Method:    MSChartAxisFormatDlg.Init
; Purpose:   Initialaizes a dialog.
; Arguments: Arg1: -> MSChart object.
;            Arg2: Axis ID.
; Return:    Nothing.

Method MSChartAxisFormatDlg.Init, uses esi, pOwner:Pointer, dAxisID:dword
    SetObject esi
    m2m [esi].dAxisID, dAxisID, eax
    mov eax, pOwner
    ACall esi.Init, eax, [eax].MSChart.hWnd, 1000
MethodEnd

; 
; Method:    MSChartAxisFormatDlg.OnCommand
; Purpose:   Event procedure for WM_COMMAND message.
; Arguments: Arg1: First message parameter.
;            Arg2: Second message parameter.
; Return:    Nothing.

Method MSChartAxisFormatDlg.OnCommand, uses esi, wParam:dword, lParam:dword
    SetObject esi
    LoWord(wParam)
    .if ax == IDOK
      OCall esi.CtlsGet
      OCall esi.DestroyDialog, wParam
    .elseif ax == IDCANCEL
      OCall esi.DestroyDialog, wParam
    .endif
MethodEnd

; 
; Method:    MSChartAxisFormatDlg.OnDestroy
; Purpose:   Event procedure for WM_DESTROY message.
; Arguments: Arg1: First message parameter.
;            Arg2: Second message parameter.
; Return:    Nothing.

Method MSChartAxisFormatDlg.OnDestroy, uses esi, wParam:dword, lParam:dword
    SetObject esi
    invoke SetWindowLong, [esi].hWnd, DWL_USER, NULL
MethodEnd


endif